//*********************************************************
//GLFONT.CPP -- glFont routines
//Copyright (c) 1998 Brad Fish
//See glFont.txt for terms of use
//November 10, 1998
//*********************************************************

#ifdef WIN32
#include <windows.h>

#endif
#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <GL/gl.h>
#include "glfont.h"
#include "popup.h"

//*********************************************************
//Variables
//*********************************************************

//Current font
GLFONT *glFont;

//*********************************************************
//Functions
//*********************************************************
char *glFontCreate(GLFONT * Font, char *FileName, int Tex, int fade, int softedges)
{
	FILE *Input;
	unsigned char *TexBytes;
	unsigned char *TexBytes2;
	int Num;
	int i, j, k;

	//Open font file
	if ((Input = fopen(FileName, "rb")) == NULL)
		return FALSE;

	//Read glFont structure
	fread(Font, sizeof(GLFONT), 1, Input);

	//Save texture number
	Font->Tex = Tex;

	//Get number of characters
	Num = Font->IntEnd - Font->IntStart + 1;

	//Allocate memory for characters
	if ((Font->Char = (GLFONTCHAR *) malloc(sizeof(GLFONTCHAR) * Num)) == NULL)
		return FALSE;

	//Read glFont characters
	fread(Font->Char, sizeof(GLFONTCHAR), Num, Input);

	//Get texture size
	Num = Font->TexWidth * Font->TexHeight * 2;

	//Allocate memory for texture data
	if ((TexBytes = (unsigned char *)malloc(Num)) == NULL)
		return FALSE;
	if ((TexBytes2 = (unsigned char *)malloc(Num)) == NULL)
		return FALSE;

	//Read texture data
	fread(TexBytes, sizeof(unsigned char), Num, Input);
	for (i = 1; i < Font->TexHeight; i++)
	{
		unsigned char v1, v2, v3, v4;
		for (j = 0; j < Font->TexWidth * 2; j++)
		{
			k = (i * Font->TexWidth * 2) + j;
			//fread(TexBytes+k, sizeof(unsigned char), 1, Input);
			if (j % 2 == 0)
			{	//intensity channel

				v1 = (i < Font->TexHeight - 1 ? TexBytes[k + Font->TexWidth * 2] : TexBytes[k]);
				v2 = (i > 0 ? TexBytes[k - 2] : TexBytes[k]);
				v3 = (j < Font->TexWidth * 2 - 1 ? TexBytes[k + 2] : TexBytes[k]);
				v4 = (j > 0 ? TexBytes[k - Font->TexWidth * 2] : TexBytes[k]);

				TexBytes2[k] = (TexBytes[k] + v1 + v2 + v3 + v4) / 5;	//average
				//TexBytes2[k] = max(max(max(max(TexBytes[k],v1),v2),v3),v4);//maximum filter
				//TexBytes2[k] = min(min(min(min(TexBytes[k],v1),v2),v3),v4);
				//TexBytes2[k] = TexBytes[k];

			} else
			{	//alpha channel
				v1 = (i < Font->TexHeight - 1 ? TexBytes[k + Font->TexWidth * 2] : TexBytes[k]);
				v2 = (i > 0 ? TexBytes[k - 2] : TexBytes[k]);
				v3 = (j < Font->TexWidth * 2 - 1 ? TexBytes[k + 2] : TexBytes[k]);
				v4 = (j > 0 ? TexBytes[k - Font->TexWidth * 2] : TexBytes[k]);

				//TexBytes2[k] = 255-((TexBytes[k]+v1+v2+v3+v4)/5)/fade;//average
				//TexBytes2[k] = 255-(max(max(max(max(TexBytes[k],v1),v2),v3),v4))/fade;//maximum filter
				if (softedges)
				{
					TexBytes2[k] = 255 - (max(max(max(max(TexBytes[k], v1), v2), v3), v4)) / fade;
				} else
				{
					if (max(max(max(max(TexBytes[k], v1), v2), v3), v4) > 128)
					{
						TexBytes2[k] = 0;
					} else
					{
						TexBytes2[k] = 255;
					}
				}

				//TexBytes2[k] = 255-(min(min(min(min(TexBytes[k],v1),v2),v3),v4))/fade;
				//TexBytes2[k] = 255-TexBytes[k]/fade;//scaled down
				//TexBytes2[k] = 255-(TexBytes[k]==255?255:0)/fade;//maximum filter

				////TexBytes[i-1] = 255;
				//if(TexBytes[i] == 0){
				//      TexBytes[i] = fade;
				//}else{
				//      TexBytes[i-1] = 0;
				//TexBytes[i] = 253;
				//}
			}
		}
	}

	//Set texture attributes
	glBindTexture(GL_TEXTURE_2D, Font->Tex);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

	//Create texture
	glTexImage2D(GL_TEXTURE_2D, 0, 2, Font->TexWidth,
		     Font->TexHeight, 0, GL_LUMINANCE_ALPHA, GL_UNSIGNED_BYTE, (void *)TexBytes2);

	//Clean up
	free(TexBytes);
	//Font->TexBytes = TexBytes;
	fclose(Input);

	//Return pointer to new font
	return (char *)TexBytes2;
}

//*********************************************************
void glFontDestroy(GLFONT * Font)
{
	//Free character memory
	free(Font->Char);
}

//*********************************************************
void glFontBegin(GLFONT * Font)
{
	//Save pointer to font structure
	if (Font->Char != NULL)
		glFont = Font;
	else
		glFont = NULL;

	//Bind to font texture
	glBindTexture(GL_TEXTURE_2D, Font->Tex);
}

//*********************************************************
void glFontEnd(void)
{
	//Font no longer current
	glFont = NULL;
}

//*********************************************************
void glFontStringSize(char *String, float maxwidth, float *x, float *y)
{
	int Length, i;
	float xincr = 0.0;
	GLFONTCHAR *Char;
	int lastspace;
	float xatlastspace;

	//printf("%f maxwidth\n",  maxwidth);
	*x = 0.0;
	*y = 2.3;		//approximate height of a line of text

	//Return if we don't have a valid glFont 
	if (glFont == NULL)
		return;

	//Get length of string
	Length = strlen(String);
	if (Length == 0)
		*y = 0.0;

	//Loop through characters
	for (i = 0; i < Length; i++)
	{

		if (String[i] > '~' || String[i] < 0)
		{
			continue;  //  http://bugzilla.treshna.com/show_bug.cgi?id=4697#c2
		}
		 else if (String[i] == ' ')
		{
			lastspace = i;
			xatlastspace = xincr;
		} else if (String[i] == '\n')
		{
			if (i == Length - 1)
				continue;
			*y += 2.5;
			*x = max(*x, xincr);
			xincr = 0.0;
			lastspace = 0;
			continue;
		}
		//Get pointer to glFont character
		Char = &glFont->Char[(int)String[i] - glFont->IntStart];

		if (xincr + Char->dx > maxwidth)
		{
			if (lastspace == 0)
			{
				//no space to back up to, easiest option for now is to keep going till we reach one
				xincr += Char->dx;
			} else
			{
				//move back to last space and replace it with a new line
				i = lastspace - 1;
				xincr = xatlastspace;
				String[lastspace] = '\n';
			}
		} else
		{
			//Move to next character
			xincr += Char->dx;
		}
	}
	*x = max(*x, xincr);
}

//*********************************************************
void glFontTextOut(char *String, float *x, float *y, float *z)
{
	int Length, i;
	float xstart = *x, xcur = *x;
	GLFONTCHAR *Char;

	//Return if we don't have a valid glFont 
	if (glFont == NULL)
		return;

	//Get length of string
	Length = strlen(String);

	//Begin rendering quads
	glBegin(GL_QUADS);

	//Loop through characters
	for (i = 0; i < Length; i++)
	{
		if (String[i] > '~' || String[i] < 0)
		{
			continue;  //  http://bugzilla.treshna.com/show_bug.cgi?id=4697#c2
		}
		else if (String[i] == '\n')
		{
			if (i == Length - 1)
				continue;
			*y -= 2.5;
			xcur = xstart;
			continue;
		}
		//Get pointer to glFont character
		Char = &glFont->Char[(int)String[i] - glFont->IntStart];

		if (String[i] != ' ')
		{
			//Specify vertices and texture coordinates
			glTexCoord2f(Char->tx1, Char->ty1);
			glVertex3f(xcur, *y, *z);
			glTexCoord2f(Char->tx1, Char->ty2);
			glVertex3f(xcur, *y - Char->dy, *z);
			glTexCoord2f(Char->tx2, Char->ty2);
			glVertex3f(xcur + Char->dx, *y - Char->dy, *z);
			glTexCoord2f(Char->tx2, Char->ty1);
			glVertex3f(xcur + Char->dx, *y, *z);
		}
		//Move to next character
		xcur += Char->dx;
	}
	*y -= 2.5;		//add a new line to y;
	//Stop rendering quads
	glEnd();
}

//*********************************************************

//End of file
